from __future__ import annotations

from typing import cast

import ckan.plugins.toolkit as tk
from ckan import model, types
from ckan.logic import validate

import ckanext.mailcraft_dashboard.model as mc_model
from ckanext.mailcraft_dashboard.logic import schema


@validate(schema.mail_create_schema)
def mc_mail_create(context: types.Context, data_dict: types.DataDict):
    tk.check_access("mc_mail_create", context, data_dict)

    email = mc_model.Email.create(**data_dict)

    return email.dictize(context)


@tk.side_effect_free
@validate(schema.mail_list_schema)
def mc_mail_list(context: types.Context, data_dict: types.DataDict):
    tk.check_access("mc_mail_list", context, data_dict)

    query = model.Session.query(mc_model.Email)

    if data_dict.get("state"):
        query = query.filter(mc_model.Email.state == data_dict["state"])

    query = query.order_by(mc_model.Email.timestamp.desc())

    return [mail.dictize(context) for mail in query.all()]


@tk.side_effect_free
@validate(schema.mail_show_schema)
def mc_mail_show(context: types.Context, data_dict: types.DataDict):
    tk.check_access("mc_mail_show", context, data_dict)

    return mc_model.Email.get(data_dict["id"]).dictize(context)  # type: ignore


@validate(schema.mail_delete_schema)
def mc_mail_delete(context: types.Context, data_dict: types.DataDict):
    tk.check_access("mc_mail_delete", context, data_dict)

    mail = cast(mc_model.Email, mc_model.Email.get(data_dict["id"]))
    mail.delete()

    model.Session.commit()

    return True


def mc_mail_clear(context: types.Context, data_dict: types.DataDict):
    """Clear all stored mails."""
    tk.check_access("mc_mail_delete", context, data_dict)

    emails_deleted = mc_model.Email.clear_emails()

    return {"deleted": emails_deleted}



from __future__ import annotations

import ckan.plugins.toolkit as tk

CONF_TEST_CONN = "ckanext.mailcraft.test_conn_on_startup"
DEF_TEST_CONN = False

CONF_CONN_TIMEOUT = "ckanext.mailcraft.conn_timeout"
DEF_CONN_TIMEOUT = 10

CONF_STOP_OUTGOING = "ckanext.mailcraft.stop_outgoing_emails"
DEF_STOP_OUTGOING = False

CONF_SAVE_EMAILS_TO_DB = "ckanext.mailcraft.save_emails"
DEF_SAVE_EMAILS_TO_DB = False

CONF_MAIL_PER_PAGE = "ckanext.mailcraft.mail_per_page"
DEF_MAIL_PER_PAGE = 20

CONF_REDIRECT_EMAILS_TO = "ckanext.mailcraft.redirect_emails_to"

CONF_STORE_ATTACHMENTS = "ckanext.mailcraft.store_attachment_content"
DEF_STORE_ATTACHMENTS = False


def get_conn_timeout() -> int:
    """Return a timeout for an SMTP connection."""
    return tk.asint(tk.config.get(CONF_CONN_TIMEOUT) or DEF_CONN_TIMEOUT)


def is_startup_conn_test_enabled() -> bool:
    """Check do we want to check an SMTP conn on CKAN startup."""
    return tk.asbool(tk.config.get(CONF_TEST_CONN, DEF_TEST_CONN))


def stop_outgoing_emails() -> bool:
    """Check if we are stopping outgoing emails.

    In this case, we are only, we are only saving it to dashboard
    """
    return tk.asbool(tk.config.get(CONF_STOP_OUTGOING, DEF_STOP_OUTGOING))


def save_emails_to_db() -> bool:
    """Check if we are saving emails to database."""
    return tk.asbool(tk.config.get(CONF_SAVE_EMAILS_TO_DB, DEF_SAVE_EMAILS_TO_DB))


def get_mail_per_page() -> int:
    """Return a number of mails to show per page."""
    return tk.asint(tk.config.get(CONF_MAIL_PER_PAGE) or DEF_MAIL_PER_PAGE)


def get_redirect_email() -> list[str]:
    """Redirect outgoing emails to a specified email."""
    return tk.config.get(CONF_REDIRECT_EMAILS_TO, [])


def store_attachments_content() -> bool:
    """Check if we are storing attachments content in the database."""
    return tk.asbool(tk.config.get(CONF_STORE_ATTACHMENTS, DEF_STORE_ATTACHMENTS))



from __future__ import annotations

from typing import TypedDict

from typing_extensions import NotRequired


class Attachment(TypedDict):
    """Attachment structure."""

    name: str
    content: bytes
    media_type: NotRequired[str | None]
    cid: NotRequired[str | None]
    disposition: NotRequired[str]


class AttachmentData(TypedDict):
    """Attachment data structure."""

    filename: str
    content_id: NotRequired[str | None]
    disposition: str
    content_type: str
    size: int
    data: NotRequired[bytes | None]


EmailData = TypedDict(
    "EmailData",
    {
        "Bcc": str,
        "Content-Type": NotRequired[str],
        "Date": str,
        "From": str,
        "MIME-Version": NotRequired[str],
        "Subject": str,
        "To": str,
        "X-Mailer": NotRequired[str],
        "redirected_from": NotRequired["list[str]"],
        "attachments": NotRequired["list[AttachmentData]"],
    },
)



from __future__ import annotations

import ckan.types as types
from ckan.logic.schema import validator_args
from ckanext.mailcraft_dashboard.model import Email


@validator_args
def mail_create_schema(
    not_empty: types.Validator,
    unicode_safe: types.Validator,
    default: types.Validator,
    json_object: types.Validator,
    ignore_missing: types.Validator,
    ignore: types.Validator,
) -> types.Schema:
    return {
        "subject": [not_empty, unicode_safe],
        "sender": [not_empty, unicode_safe],
        "recipient": [not_empty, unicode_safe],
        "message": [not_empty, unicode_safe],
        "state": [default(Email.State.success), unicode_safe],  # type: ignore
        "extras": [ignore_missing, json_object],
        "__extras": [ignore],
    }


@validator_args
def mail_list_schema() -> types.Schema:
    return {}


@validator_args
def mail_show_schema(
    not_empty: types.Validator,
    unicode_safe: types.Validator,
    mc_mail_exists: types.Validator,
) -> types.Schema:
    return {"id": [not_empty, unicode_safe, mc_mail_exists]}


@validator_args
def mail_delete_schema(
    not_empty: types.Validator,
    unicode_safe: types.Validator,
    mc_mail_exists: types.Validator,
) -> types.Schema:
    return {"id": [not_empty, unicode_safe, mc_mail_exists]}



import pytest
from faker import Faker
from pytest_factoryboy import register
from factory.declarations import LazyFunction

from ckan.tests import factories

from ckanext.mailcraft_dashboard.model import Email

fake = Faker()


@register(_name="email")
class EmailFactory(factories.CKANFactory):
    class Meta:  # type: ignore
        model = Email
        action = "mc_mail_create"

    subject = LazyFunction(lambda: fake.sentence())
    timestamp = LazyFunction(lambda: fake.date())
    sender = LazyFunction(lambda: fake.email())
    recipient = LazyFunction(lambda: fake.email())
    message = LazyFunction(lambda: fake.text())
    state = LazyFunction(
        lambda: fake.random_element(
            elements=[
                Email.State.success,
                Email.State.failed,
                Email.State.stopped,
            ]
        )
    )
    extras = {"key": "value"}


@pytest.fixture()
def clean_db(reset_db, migrate_db_for):
    reset_db()

    migrate_db_for("mailcraft_dashboard")


@register(_name="sysadmin")
class SysadminFactory(factories.Sysadmin):
    pass



# encoding: utf-8

import base64
import email.utils
import io
from email.header import decode_header
from email.mime.text import MIMEText
from email.parser import Parser

import pytest

import ckan.lib.helpers as h
import ckan.lib.mailer as mailer
import ckan.model as model
import ckan.tests.factories as factories
from ckan.common import config


class MailerBase(object):
    def mime_encode(self, msg, recipient_name, subtype="plain"):
        text = MIMEText(msg.encode("utf-8"), subtype, "utf-8")
        return text.get_payload().strip()

    def get_email_body(self, msg):
        payload = Parser().parsestr(msg).get_payload()
        return base64.b64decode(payload)

    def get_email_subject(self, msg):
        header = Parser().parsestr(msg)["Subject"]
        return decode_header(header)[0][0]


@pytest.mark.usefixtures("non_clean_db")
class TestMailer(MailerBase):
    def test_mail_recipient(self, mail_server):
        user = factories.User()

        msgs = mail_server.get_smtp_messages()
        assert msgs == []

        # send email
        test_email = {
            "recipient_name": "Bob",
            "recipient_email": user["email"],
            "subject": "Meeting",
            "body": "The meeting is cancelled.\n",
            "headers": {"header1": "value1"},
        }
        mailer.mail_recipient(**test_email)

        # check it went to the mock smtp server
        msgs = mail_server.get_smtp_messages()
        assert len(msgs) == 1
        msg = msgs[0]
        assert msg[1] == config["smtp.mail_from"]
        assert msg[2] == [test_email["recipient_email"]]
        assert list(test_email["headers"].keys())[0] in msg[3], msg[3]
        assert list(test_email["headers"].values())[0] in msg[3], msg[3]
        assert test_email["subject"] in msg[3], msg[3]
        assert "X-Mailer" in msg[3], "Missing X-Mailer header"
        expected_body = self.mime_encode(
            test_email["body"], test_email["recipient_name"]
        )
        assert expected_body in msg[3]

    @pytest.mark.ckan_config("ckan.hide_version", True)
    def test_mail_recipient_hiding_mailer(self, mail_server):
        user = factories.User()

        msgs = mail_server.get_smtp_messages()
        assert msgs == []

        # send email
        test_email = {
            "recipient_name": "Bob",
            "recipient_email": user["email"],
            "subject": "Meeting",
            "body": "The meeting is cancelled.\n",
            "headers": {"header1": "value1"},
        }
        mailer.mail_recipient(**test_email)

        # check it went to the mock smtp server
        msgs = mail_server.get_smtp_messages()
        assert len(msgs) == 1
        msg = msgs[0]
        assert msg[1] == config["smtp.mail_from"]
        assert msg[2] == [test_email["recipient_email"]]
        assert list(test_email["headers"].keys())[0] in msg[3], msg[3]
        assert list(test_email["headers"].values())[0] in msg[3], msg[3]
        assert test_email["subject"] in msg[3], msg[3]
        assert msg[3].startswith("Content-Type: text/plain"), msg[3]
        assert "X-Mailer" not in msg[3], "Should have skipped X-Mailer header"
        expected_body = self.mime_encode(
            test_email["body"], test_email["recipient_name"]
        )
        assert expected_body in msg[3]

    def test_mail_recipient_with_html(self, mail_server):
        user = factories.User()

        msgs = mail_server.get_smtp_messages()
        assert msgs == []

        # send email
        test_email = {
            "recipient_name": "Bob",
            "recipient_email": user["email"],
            "subject": "Meeting",
            "body": "The meeting is cancelled.\n",
            "body_html": 'The <a href="meeting">meeting</a> is cancelled.\n',
            "headers": {"header1": "value1"},
        }
        mailer.mail_recipient(**test_email)

        # check it went to the mock smtp server
        msgs = mail_server.get_smtp_messages()
        assert len(msgs) == 1
        msg = msgs[0]
        assert msg[1] == config["smtp.mail_from"]
        assert msg[2] == [test_email["recipient_email"]]
        assert list(test_email["headers"].keys())[0] in msg[3], msg[3]
        assert list(test_email["headers"].values())[0] in msg[3], msg[3]
        assert test_email["subject"] in msg[3], msg[3]
        assert "Content-Type: multipart" in msg[3]
        expected_plain_body = self.mime_encode(
            test_email["body"], test_email["recipient_name"], subtype="plain"
        )
        assert expected_plain_body in msg[3]
        expected_html_body = self.mime_encode(
            test_email["body_html"], test_email["recipient_name"], subtype="html"
        )
        assert expected_html_body in msg[3]

    def test_mail_user(self, mail_server):
        user = factories.User()
        user_obj = model.User.by_name(user["name"])

        msgs = mail_server.get_smtp_messages()
        assert msgs == []

        # send email
        test_email = {
            "recipient": user_obj,
            "subject": "Meeting",
            "body": "The meeting is cancelled.\n",
            "headers": {"header1": "value1"},
        }
        mailer.mail_user(**test_email)

        # check it went to the mock smtp server
        msgs = mail_server.get_smtp_messages()
        assert len(msgs) == 1
        msg = msgs[0]
        assert msg[1] == config["smtp.mail_from"]
        assert msg[2] == [user["email"]]
        assert list(test_email["headers"].keys())[0] in msg[3], msg[3]
        assert list(test_email["headers"].values())[0] in msg[3], msg[3]
        assert test_email["subject"] in msg[3], msg[3]
        expected_body = self.mime_encode(test_email["body"], user["name"])

        assert expected_body in msg[3]

    def test_mail_user_without_email(self):
        # send email
        mary = model.User(email=None)
        # model.Session.add(mary)
        # model.Session.commit()

        test_email = {
            "recipient": mary,
            "subject": "Meeting",
            "body": "The meeting is cancelled.",
            "headers": {"header1": "value1"},
        }
        with pytest.raises(mailer.MailerException):
            mailer.mail_user(**test_email)

    @pytest.mark.ckan_config("ckan.site_title", "My CKAN instance")
    def test_from_field_format(self, mail_server):
        msgs = mail_server.get_smtp_messages()
        assert msgs == []

        # send email
        test_email = {
            "recipient_name": "Bob",
            "recipient_email": "Bob@bob.com",
            "subject": "Meeting",
            "body": "The meeting is cancelled.",
            "headers": {"header1": "value1"},
        }
        mailer.mail_recipient(**test_email)

        # check it went to the mock smtp server
        msgs = mail_server.get_smtp_messages()
        msg = msgs[0]

        expected_from_header = email.utils.formataddr(
            (config.get("ckan.site_title"), config.get("smtp.mail_from"))
        )

        assert expected_from_header in msg[3]

    @pytest.mark.usefixtures("with_request_context")
    def test_send_reset_email(self, mail_server):
        user = factories.User()
        user_obj = model.User.by_name(user["name"])

        mailer.send_reset_link(user_obj)

        # check it went to the mock smtp server
        msgs = mail_server.get_smtp_messages()
        assert len(msgs) == 1
        msg = msgs[0]
        assert msg[1] == config["smtp.mail_from"]
        assert msg[2] == [user["email"]]
        assert "Reset" in msg[3], msg[3]
        test_msg = mailer.get_reset_link_body(user_obj)
        expected_body = self.mime_encode(test_msg + "\n", user["name"])

        assert expected_body in msg[3]

    @pytest.mark.usefixtures("with_request_context")
    def test_send_invite_email(self, mail_server):
        user = factories.User()
        user_obj = model.User.by_name(user["name"])
        assert user_obj.reset_key is None, user_obj

        # send email
        mailer.send_invite(user_obj)

        # check it went to the mock smtp server
        msgs = mail_server.get_smtp_messages()
        assert len(msgs) == 1
        msg = msgs[0]
        assert msg[1] == config["smtp.mail_from"]
        assert msg[2] == [user["email"]]
        test_msg = mailer.get_invite_body(user_obj)
        expected_body = self.mime_encode(test_msg + "\n", user["name"])

        assert expected_body in msg[3]
        assert user_obj.reset_key is not None, user

    @pytest.mark.usefixtures("with_request_context")
    def test_send_invite_email_with_group(self, mail_server):
        user = factories.User()
        user_obj = model.User.by_name(user["name"])

        group = factories.Group()
        role = "member"

        # send email
        mailer.send_invite(user_obj, group_dict=group, role=role)

        # check it went to the mock smtp server
        msgs = mail_server.get_smtp_messages()
        msg = msgs[0]
        body = self.get_email_body(msg[3])
        assert group["title"] in body.decode()
        assert h.roles_translated()[role] in body.decode()

    @pytest.mark.usefixtures("with_request_context")
    def test_send_invite_email_with_org(self, mail_server):
        user = factories.User()
        user_obj = model.User.by_name(user["name"])

        org = factories.Organization()
        role = "admin"

        # send email
        mailer.send_invite(user_obj, group_dict=org, role=role)

        # check it went to the mock smtp server
        msgs = mail_server.get_smtp_messages()
        msg = msgs[0]
        body = self.get_email_body(msg[3])
        assert org["title"] in body.decode()
        assert h.roles_translated()[role] in body.decode()

    @pytest.mark.ckan_config("smtp.server", "999.999.999.999")
    def test_bad_smtp_host(self):
        test_email = {
            "recipient_name": "Bob",
            "recipient_email": "b@example.com",
            "subject": "Meeting",
            "body": "The meeting is cancelled.",
            "headers": {"header1": "value1"},
        }
        with pytest.raises(mailer.MailerException):
            mailer.mail_recipient(**test_email)

    @pytest.mark.ckan_config("smtp.reply_to", "norply@ckan.org")
    def test_reply_to(self, mail_server):
        msgs = mail_server.get_smtp_messages()
        assert msgs == []

        # send email
        test_email = {
            "recipient_name": "Bob",
            "recipient_email": "Bob@bob.com",
            "subject": "Meeting",
            "body": "The meeting is cancelled.",
            "headers": {"header1": "value1"},
        }
        mailer.mail_recipient(**test_email)

        # check it went to the mock smtp server
        msgs = mail_server.get_smtp_messages()
        msg = msgs[0]

        expected_from_header = "Reply-to: {}".format(config.get("smtp.reply_to"))

        assert expected_from_header in msg[3]

    @pytest.mark.ckan_config("smtp.reply_to", "norply@ckan.org")
    def test_reply_to_ext_headers_overwrite(self, mail_server):
        msgs = mail_server.get_smtp_messages()
        assert msgs == []

        # send email
        test_email = {
            "recipient_name": "Bob",
            "recipient_email": "Bob@bob.com",
            "subject": "Meeting",
            "body": "The meeting is cancelled.",
            "headers": {"Reply-to": "norply@ckanext.org"},
        }
        mailer.mail_recipient(**test_email)

        # check it went to the mock smtp server
        msgs = mail_server.get_smtp_messages()
        msg = msgs[0]

        expected_from_header = "Reply-to: norply@ckanext.org"

        assert expected_from_header in msg[3]

    def test_mail_user_with_attachments(self, mail_server):
        user = factories.User()
        user_obj = model.User.by_name(user["name"])

        msgs = mail_server.get_smtp_messages()
        assert msgs == []

        # send email
        test_email = {
            "recipient": user_obj,
            "subject": "Meeting",
            "body": "The meeting is cancelled.\n",
            "headers": {"header1": "value1"},
            "attachments": [
                ("strategy.pdf", io.BytesIO(b"Some fake pdf"), "application/pdf"),
                ("goals.png", io.BytesIO(b"Some fake png"), "image/png"),
            ],
        }
        mailer.mail_user(**test_email)

        # check it went to the mock smtp server
        msgs = mail_server.get_smtp_messages()
        assert len(msgs) == 1
        msg = msgs[0]
        assert msg[1] == config["smtp.mail_from"]
        assert msg[2] == [user["email"]]
        assert list(test_email["headers"].keys())[0] in msg[3], msg[3]
        assert list(test_email["headers"].values())[0] in msg[3], msg[3]
        assert test_email["subject"] in msg[3], msg[3]

        for item in [
            "strategy.pdf",
            base64.b64encode(b"Some fake pdf").decode(),
            "application/pdf",
            "goals.png",
            base64.b64encode(b"Some fake png").decode(),
            "image/png",
        ]:
            assert item in msg[3]

    def test_mail_user_with_attachments_no_media_type_provided(self, mail_server):
        user = factories.User()
        user_obj = model.User.by_name(user["name"])

        msgs = mail_server.get_smtp_messages()
        assert msgs == []

        # send email
        test_email = {
            "recipient": user_obj,
            "subject": "Meeting",
            "body": "The meeting is cancelled.\n",
            "headers": {"header1": "value1"},
            "attachments": [
                ("strategy.pdf", io.BytesIO(b"Some fake pdf")),
                ("goals.png", io.BytesIO(b"Some fake png")),
            ],
        }
        mailer.mail_user(**test_email)

        # check it went to the mock smtp server
        msgs = mail_server.get_smtp_messages()
        assert len(msgs) == 1
        msg = msgs[0]

        for item in [
            "strategy.pdf",
            "application/pdf",
            "goals.png",
            "image/png",
        ]:
            assert item in msg[3]



from __future__ import annotations

from typing import Any

import ckan.plugins.toolkit as tk

import ckanext.mailcraft_dashboard.model as mc_model


def mc_mail_exists(v: str, context) -> Any:
    """Ensures that the mail with a given id exists"""

    result = mc_model.Email.get(v)

    if not result:
        raise tk.Invalid(f"The ьфшд with an id {v} doesn't exist.")

    return v



"""Add mailcraft_mail table.

Revision ID: 9f83e5bfff72
Revises:
Create Date: 2023-09-14 11:53:58.371935

"""

import sqlalchemy as sa
from alembic import op
from sqlalchemy.dialects import postgresql

# revision identifiers, used by Alembic.
revision = "9f83e5bfff72"
down_revision = None
branch_labels = None
depends_on = None


def upgrade():
    op.create_table(
        "mailcraft_mail",
        sa.Column("id", sa.Integer, primary_key=True, autoincrement=True),
        sa.Column("subject", sa.Text),
        sa.Column(
            "timestamp",
            sa.DateTime,
            nullable=False,
            server_default=sa.func.current_timestamp(),
        ),
        sa.Column("sender", sa.Text),
        sa.Column("recipient", sa.Text),
        sa.Column("message", sa.Text),
        sa.Column("state", sa.Text, server_default="success"),
        sa.Column("extras", postgresql.JSONB(astext_type=sa.Text()), nullable=True),
    )


def downgrade():
    op.drop_table("mailcraft_mail")



from __future__ import annotations

from typing import Any

import ckan.plugins as p
import ckan.plugins.toolkit as tk
from ckan import types
from ckan.common import CKANConfig

from ckanext.mailcraft_dashboard.table import DashboardTable


@tk.blanket.blueprints
@tk.blanket.actions
@tk.blanket.auth_functions
@tk.blanket.validators
class MailcraftDashboardPlugin(p.SingletonPlugin):
    p.implements(p.IConfigurer)
    p.implements(p.ISignal)

    # IConfigurer

    def update_config(self, config_: CKANConfig):
        tk.add_template_directory(config_, "templates")

    # # ISignal

    def get_signal_subscriptions(self) -> types.SignalMapping:
        return {
            tk.signals.ckanext.signal("ckanext.tables.register_tables"): [
                self.collect_tables
            ],
        }

    def collect_tables(self, sender: None) -> dict[str, type[Any]]:
        return {"mailcraft": DashboardTable}



import ckan.plugins.toolkit as tk

from ckanext.tables.shared import FormatterResult, Options, Value, formatters


class StatusFormatter(formatters.BaseFormatter):
    """Formatter for the status column."""

    def format(self, value: Value, options: Options) -> FormatterResult:
        """Format the status value."""
        state_map = {"success": "success", "failed": "danger", "stopped": "black"}

        label_type = state_map.get(value, "black")

        return tk.literal(
            f'<span class="text-uppercase px-2 py-1 badge bg-{label_type}">{value}</span>'
        )



from __future__ import annotations

from sqlalchemy import select

import ckan.plugins.toolkit as tk
from ckan.types import Context

from ckanext.tables.shared import (
    BulkActionDefinition,
    BulkActionHandlerResult,
    ColumnDefinition,
    DatabaseDataSource,
    Row,
    RowActionDefinition,
    RowActionHandlerResult,
    TableActionDefinition,
    TableActionHandlerResult,
    TableDefinition,
    formatters,
)

from ckanext.mailcraft.utils import get_mailer
from ckanext.mailcraft_dashboard.formatters import StatusFormatter
from ckanext.mailcraft_dashboard.model import Email


class DashboardTable(TableDefinition):
    """Table definition for the mailcraft dashboard."""

    def __init__(self):
        """Initialize the table definition."""
        super().__init__(
            name="mailcraft",
            data_source=DatabaseDataSource(
                stmt=select(
                    Email.id,
                    Email.subject,
                    Email.sender,
                    Email.recipient,
                    Email.state,
                    Email.timestamp,
                ).order_by(Email.timestamp.desc()),
                model=Email,
            ),
            columns=[
                ColumnDefinition(field="subject", width=250),
                ColumnDefinition(field="sender"),
                ColumnDefinition(field="recipient"),
                ColumnDefinition(
                    field="state",
                    resizable=False,
                    width=100,
                    tabulator_formatter="html",
                    formatters=[(StatusFormatter, {})],
                ),
                ColumnDefinition(
                    field="timestamp",
                    formatters=[
                        (
                            formatters.DateFormatter,
                            {"date_format": "%Y-%m-%dT%H:%M:%S"},
                        ),
                    ],
                    tabulator_formatter="html",
                    resizable=False,
                    width=170,
                ),
            ],
            row_actions=[
                RowActionDefinition(
                    action="view",
                    label=tk._("View"),
                    icon="fa fa-eye",
                    callback=self.row_action_view,
                ),
                RowActionDefinition(
                    action="delete",
                    label=tk._("Delete"),
                    icon="fa fa-trash",
                    callback=self.row_action_delete,
                    with_confirmation=True
                ),
            ],
            bulk_actions=[
                BulkActionDefinition(
                    action="delete",
                    label=tk._("Delete selected entities"),
                    icon="fa fa-trash",
                    callback=self.bulk_action_remove_emails,
                )
            ],
            table_actions=[
                TableActionDefinition(
                    action="clear_emails",
                    label=tk._("Clear mails"),
                    icon="fa fa-trash",
                    callback=self.table_action_clear_emails,
                ),
                TableActionDefinition(
                    action="send_test_mail",
                    label=tk._("Send test mail"),
                    icon="fa fa-paper-plane",
                    callback=self.table_action_send_test_mail,
                ),
            ],
        )

    @staticmethod
    def row_action_view(row: Row) -> RowActionHandlerResult:
        return RowActionHandlerResult(
            success=True,
            error=None,
            redirect=tk.h.url_for("mailcraft.mail_read", mail_id=row["id"]),
        )

    @staticmethod
    def row_action_delete(row: Row) -> RowActionHandlerResult:
        try:
            tk.get_action("mc_mail_delete")(
                {"ignore_auth": True},
                {"id": row["id"]},
            )
        except tk.ObjectNotFound:
            return RowActionHandlerResult(success=False, error=tk._("Mail not found"))

        return RowActionHandlerResult(success=True, error=None)

    @staticmethod
    def bulk_action_remove_emails(row: Row) -> BulkActionHandlerResult:
        try:
            tk.get_action("mc_mail_delete")(
                {"ignore_auth": True},
                {"id": row["id"]},
            )
        except tk.ObjectNotFound:
            return False, tk._("Mail not found")

        return True, None

    def table_action_clear_emails(self) -> TableActionHandlerResult:
        try:
            tk.get_action("mc_mail_clear")({"ignore_auth": True}, {})
        except tk.ValidationError as e:
            return False, str(e)

        return True, None

    def table_action_send_test_mail(self) -> TableActionHandlerResult:
        """Send a test email and redirect to the dashboard."""
        mailer = get_mailer()

        result = mailer.mail_recipients(
            subject="Hello world",
            recipients=["test@gmail.com"],
            body="Hello world",
            body_html=tk.render(
                "mailcraft/emails/test.html",
                extra_vars={
                    "site_url": mailer.site_url,
                    "site_title": mailer.site_title,
                },
            ),
        )

        if not result:
            return False, tk._("Failed to send test email. Check your mail settings.")

        return True, None

    @classmethod
    def check_access(cls, context: Context) -> None:
        tk.check_access("sysadmin", context)



from __future__ import annotations

from flask import Blueprint
from flask.views import MethodView

import ckan.plugins.toolkit as tk
from ckan import types

from ckanext.tables.shared import GenericTableView

mailcraft = Blueprint("mailcraft", __name__, url_prefix="/ckan-admin/mailcraft")


def before_request() -> None:
    try:
        tk.check_access("sysadmin", {"user": tk.current_user.name})
    except tk.NotAuthorized:
        tk.abort(403, tk._("Need to be system administrator to administer"))


class MailReadView(MethodView):
    """View for reading a single email."""

    def get(self, mail_id: str) -> str:
        """Render the email reading template."""
        try:
            mail = tk.get_action("mc_mail_show")(_build_context(), {"id": mail_id})
        except tk.ValidationError:
            return tk.render("mailcraft/404.html")

        return tk.render("mailcraft/mail_read.html", extra_vars={"mail": mail})


def _build_context() -> types.Context:
    return {
        "user": tk.current_user.name,
        "auth_user_obj": tk.current_user,
    }


mailcraft.before_request(before_request)

mailcraft.add_url_rule("/dashboard", view_func=GenericTableView.as_view("dashboard", table="mailcraft"))
mailcraft.add_url_rule("/dashboard/read/<mail_id>", view_func=MailReadView.as_view("mail_read"))
